"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModel = void 0;
var setModelName_1 = require("../utils/setModelName");
var NormalAction_1 = require("../actions/NormalAction");
var setModel_1 = require("../utils/setModel");
var ComposeAction_1 = require("../actions/ComposeAction");
var HttpServiceBuilder_1 = require("../services/HttpServiceBuilder");
var method_1 = require("../utils/method");
var BaseReducer_1 = require("../reducers/BaseReducer");
var ForgetRegisterError_1 = require("../exceptions/ForgetRegisterError");
var NullReducerError_1 = require("../exceptions/NullReducerError");
var StoreHelper_1 = require("../stores/StoreHelper");
var BaseModel = /** @class */ (function () {
    function BaseModel(alias) {
        var _this = this;
        if (alias === void 0) { alias = ''; }
        setModel_1.setCurrentModel(this);
        this.__alias = alias;
        this.__name = this.getReducerName();
        if (this.autoRegister()) {
            StoreHelper_1.storeHelper.appendReducers(this.register());
        }
        StoreHelper_1.storeHelper.listenOnce(function (item) { return _this.onStoreCreated(item.store); });
    }
    BaseModel.prototype.getReducerName = function () {
        return this.__name || setModelName_1.setInstanceName(this.constructor.name, this.__alias);
    };
    Object.defineProperty(BaseModel.prototype, "data", {
        get: function () {
            var data = StoreHelper_1.storeHelper.getState()[this.__name];
            if (data === undefined) {
                if (this.__getInitData() === null) {
                    throw new NullReducerError_1.NullReducerError(this.__name);
                }
                else {
                    throw new ForgetRegisterError_1.ForgetRegisterError(this.__name);
                }
            }
            return data;
        },
        enumerable: false,
        configurable: true
    });
    BaseModel.prototype.reset = function () {
        var _this = this;
        this.changeReducer(function () {
            return _this.initReducer();
        });
    };
    BaseModel.prototype.changeReducer = function (fn) {
        // Make sure reducer is registered and initData not null.
        this.data;
        if (this.__anonymousAction) {
            this.__anonymousAction.changeCallback(fn);
        }
        else {
            this.__anonymousAction = this.action(fn);
            this.__anonymousAction.setName('anonymous-action');
        }
        this.__anonymousAction();
    };
    BaseModel.prototype.action = function (effect) {
        var action = new NormalAction_1.NormalAction(this, effect);
        return action;
    };
    BaseModel.prototype.compose = function (fn) {
        var action = new ComposeAction_1.ComposeAction(this, fn);
        return action;
    };
    BaseModel.prototype.effects = function () {
        return [];
    };
    BaseModel.prototype.get = function (uri) {
        return this.__createHttpServiceBuilder(uri, method_1.METHOD.get);
    };
    BaseModel.prototype.post = function (uri) {
        return this.__createHttpServiceBuilder(uri, method_1.METHOD.post);
    };
    BaseModel.prototype.put = function (uri) {
        return this.__createHttpServiceBuilder(uri, method_1.METHOD.put);
    };
    BaseModel.prototype.delete = function (uri) {
        return this.__createHttpServiceBuilder(uri, method_1.METHOD.delete);
    };
    BaseModel.prototype.patch = function (uri) {
        return this.__createHttpServiceBuilder(uri, method_1.METHOD.patch);
    };
    BaseModel.prototype.connect = function (uri) {
        return this.__createHttpServiceBuilder(uri, method_1.METHOD.connect);
    };
    BaseModel.prototype.register = function () {
        var reducer = new BaseReducer_1.BaseReducer(this, this.__getInitData());
        return reducer.createReducer();
    };
    BaseModel.prototype.autoRegister = function () {
        return true;
    };
    BaseModel.prototype.onStoreCreated = function (
    // @ts-ignore
    store) { };
    BaseModel.prototype.__createHttpServiceBuilder = function (uri, method) {
        var builder = new HttpServiceBuilder_1.HttpServiceBuilder({
            uri: uri,
            method: method,
            instanceName: this.__name,
        });
        // @ts-ignore
        // @ts-expect-error
        return builder;
    };
    BaseModel.prototype.__getInitData = function () {
        return this.__initData === undefined ? this.initReducer() : this.__initData;
    };
    return BaseModel;
}());
exports.BaseModel = BaseModel;
