import { HttpServiceBuilderWithMeta, HttpServiceBuilderWithMetas, HttpServiceBuilder } from './HttpServiceBuilder';
import { IResponseAction, BaseRequestAction, IBaseRequestAction, InternalSuccessAction } from '../actions/BaseRequestAction';
import { Middleware, Action } from 'redux';
import { OrphanRequestOptions } from './OrphanHttpService';
export interface FetchHandle<Response = any, Payload = any, CancelFn = () => void> extends Promise<IResponseAction<Response, Payload>> {
    cancel: CancelFn;
}
export declare type PickData<T> = T extends (...args: any[]) => HttpServiceBuilder<infer Data, any, any, any, any> ? Data : never;
export declare type PickResponse<T> = T extends (...args: any[]) => HttpServiceBuilder<any, infer Response, any, any, any> ? Response : never;
export declare type PickPayload<T> = T extends (...args: any[]) => HttpServiceBuilder<any, any, infer Payload, any, any> ? Payload : never;
export declare type PickMeta<T> = T extends (...args: any[]) => HttpServiceBuilderWithMetas<any, any, any, any, infer M> ? M : never;
export interface BaseHttpServiceConfig {
    baseUrl: string;
    requestConfig?: object;
    onShowSuccess: (successText: string, action: IResponseAction) => void;
    onShowError: (errorText: string, action: IResponseAction) => void;
    timeoutMessage?: (originalText: string) => string;
    networkErrorMessage?: (originalText: string) => string;
}
export declare const CLEAR_THROTTLE = "@flus/clear/throttle";
export interface IClearThrottleAction extends Action<string> {
    uniqueId: number;
    key: string;
}
export declare abstract class BaseHttpService<T extends BaseHttpServiceConfig, CancelFn> {
    protected readonly uniqueId: number;
    protected readonly config: T;
    protected caches: Partial<{
        [key: string]: Partial<{
            [key: string]: {
                timestamp: number;
                response: any;
            };
        }>;
    }>;
    constructor(config: T);
    action<Fn extends (...args: any[]) => HttpServiceBuilderWithMeta<Data, Response, Payload, any>, Data = PickData<Fn>, Response = PickResponse<Fn>, Payload = PickPayload<Fn>>(fn: Fn): ((...args: Parameters<Fn>) => FetchHandle<Response, Payload, CancelFn>) & Omit<BaseRequestAction<Data, Fn, Response, Payload, true>, 'metas' | 'loadings'>;
    action<Fn extends (...args: any[]) => HttpServiceBuilderWithMetas<Data, Response, Payload, any, M>, Data = PickData<Fn>, Response = PickResponse<Fn>, Payload = PickPayload<Fn>, M = PickMeta<Fn>>(fn: Fn): ((...args: Parameters<Fn>) => FetchHandle<Response, Payload, CancelFn>) & Omit<BaseRequestAction<Data, Fn, Response, Payload, M>, 'meta' | 'loading'>;
    getAsync<Response>(config: OrphanRequestOptions<T['requestConfig']>): FetchHandle<Response, never, CancelFn>;
    postAsync<Response>(config: OrphanRequestOptions<T['requestConfig']>): FetchHandle<Response, never, CancelFn>;
    putAsync<Response>(config: OrphanRequestOptions<T['requestConfig']>): FetchHandle<Response, never, CancelFn>;
    deleteAsync<Response>(config: OrphanRequestOptions<T['requestConfig']>): FetchHandle<Response, never, CancelFn>;
    patchAsync<Response>(config: OrphanRequestOptions<T['requestConfig']>): FetchHandle<Response, never, CancelFn>;
    connectAsync<Response>(config: OrphanRequestOptions<T['requestConfig']>): FetchHandle<Response, never, CancelFn>;
    protected generateThrottleKey(action: IBaseRequestAction): string;
    protected isFetchAction(action: IBaseRequestAction | InternalSuccessAction | IClearThrottleAction): action is IBaseRequestAction;
    protected isClearAction(action: IBaseRequestAction | InternalSuccessAction | IClearThrottleAction): action is IClearThrottleAction;
    protected createThrottleMiddleware(): Middleware;
    protected createRequestMiddleware(): Middleware;
    protected abstract runAction(action: any): Promise<any>;
    protected triggerShowSuccess(okResponse: InternalSuccessAction, successText: string): void;
    protected triggerShowError(errorResponse: InternalSuccessAction, hideError: boolean | ((response: InternalSuccessAction) => boolean)): void;
}
