"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpService = void 0;
var tslib_1 = require("tslib");
var axios_1 = tslib_1.__importDefault(require("axios"));
var core_1 = require("../core");
var RequestAction_1 = require("../actions/RequestAction");
var HttpService = /** @class */ (function (_super) {
    tslib_1.__extends(HttpService, _super);
    function HttpService(config) {
        var _this = _super.call(this, config) || this;
        _this.httpHandler = axios_1.default.create(tslib_1.__assign({ baseURL: config.baseUrl, timeout: 20000, withCredentials: false, responseType: 'json' }, config.requestConfig));
        return _this;
    }
    HttpService.prototype.clone = function (config) {
        return new HttpService(tslib_1.__assign(tslib_1.__assign({}, this.config), config));
    };
    HttpService.prototype.action = function (fn) {
        return new RequestAction_1.RequestAction(fn, this.uniqueId);
    };
    HttpService.prototype.runAction = function (action) {
        var _this = this;
        var _a, _b;
        (_b = (_a = this.config).beforeSend) === null || _b === void 0 ? void 0 : _b.call(_a, action);
        // For service.xxxAsync(), prepare and fail is empty string.
        var _c = action.type, prepare = _c.prepare, success = _c.success, fail = _c.fail;
        var source = axios_1.default.CancelToken.source();
        var requestOptions = tslib_1.__assign(tslib_1.__assign({ url: action.uri, params: action.query, cancelToken: source.token, method: action.method }, action.requestOptions), { headers: tslib_1.__assign(tslib_1.__assign({}, this.config.headers(action)), action.requestOptions.headers) });
        if (action.method !== core_1.METHOD.get && action.body) {
            requestOptions.data = action.body;
        }
        var successInvoked = false;
        prepare && core_1.storeHelper.dispatch(tslib_1.__assign(tslib_1.__assign({}, action), { type: prepare, loading: true, effect: action.onPrepare }));
        var promise = this.httpHandler.request(requestOptions)
            .then(function (response) {
            if (_this.config.isSuccess && !_this.config.isSuccess(response)) {
                return Promise.reject({
                    response: response,
                });
            }
            if (_this.config.transformSuccessData) {
                response.data = _this.config.transformSuccessData(response.data, response.headers);
            }
            var okAction = tslib_1.__assign(tslib_1.__assign({}, action), { type: success, loading: false, response: response.data, effect: action.onSuccess });
            successInvoked = true;
            success && core_1.storeHelper.dispatch(okAction);
            _this.triggerShowSuccess(okAction, action.successText);
            return Promise.resolve(okAction);
        })
            .catch(function (error) {
            if (successInvoked) {
                return Promise.reject(error);
            }
            var isCancel = axios_1.default.isCancel(error);
            var errorMessage;
            var httpStatus;
            var businessCode;
            if (isCancel) {
                errorMessage = error.message || 'Abort';
            }
            else if (error.response) {
                var transform = {
                    httpStatus: error.response.status,
                };
                _this.config.onRespondError(error.response, transform);
                errorMessage = action.failText || transform.message || 'Fail to request api';
                httpStatus = transform.httpStatus;
                businessCode = transform.businessCode;
            }
            else {
                errorMessage = error.message;
                if (/^timeout\sof\s\d+m?s\sexceeded$/i.test(errorMessage)) {
                    errorMessage = _this.config.timeoutMessage ? _this.config.timeoutMessage(errorMessage) : errorMessage;
                }
                else if (/Network\sError/i.test(errorMessage)) {
                    errorMessage = _this.config.networkErrorMessage ? _this.config.networkErrorMessage(errorMessage) : errorMessage;
                }
            }
            var errorResponse = tslib_1.__assign(tslib_1.__assign({}, action), { response: error.response, type: fail, loading: false, message: errorMessage, httpStatus: httpStatus,
                businessCode: businessCode, effect: action.onFail });
            fail && core_1.storeHelper.dispatch(errorResponse);
            if (!isCancel) {
                _this.triggerShowError(errorResponse, action.hideError);
            }
            return Promise.reject(errorResponse);
        });
        // @ts-ignore
        // @ts-expect-error
        var fakePromise = promise;
        fakePromise.cancel = source.cancel;
        return fakePromise;
    };
    return HttpService;
}(core_1.BaseHttpService));
exports.HttpService = HttpService;
